void handleTelnet(){
  // Cleanup disconnected session
  for(telnetClients = 0; telnetClients < MAX_TELNET_CLIENTS; telnetClients++){
    if (TelnetClient[telnetClients] && !TelnetClient[telnetClients].connected()){
      Serial.print("Client disconnected ... terminate session "); 
      Serial.println(telnetClients+1); 
      TelnetClient[telnetClients].stop();
    }
  }
  
  // Check new client connections
  if(TelnetServer.hasClient()){
    ConnectionEstablished = false;
    
    for(telnetClients = 0; telnetClients < MAX_TELNET_CLIENTS; telnetClients++){
      
      // find free socket
      if(!TelnetClient[telnetClients]){
        TelnetClient[telnetClients] = TelnetServer.available(); 
        
        Serial.print("New Telnet client connected to session "); 
        Serial.println(telnetClients+1);
        
        uint8_t actualTelnetClient = telnetClients; //Durch das Senden einer Nachricht wird die globale Variable telnetClients verändert, dadurch können keine gezielten Nachrichten mehr verschickt werden  
        TelnetClient[actualTelnetClient].flush();  // clear input buffer, else you get strange characters
        TelnetClient[actualTelnetClient].println("----------------------------------------------------------------");
        sendWiFiInformation();
        TelnetClient[actualTelnetClient].println();
        sendMQTTInformation();
        TelnetClient[actualTelnetClient].println("--------------Verbindung wird geschlossen-----------------------");
        
        ConnectionEstablished = true; 

        TelnetClient[actualTelnetClient].stop(); //Infos senden und Sitzung gleich wieder beenden
        
        break;
      }
      
    }

    if(ConnectionEstablished == false){
      Serial.println("No free sessions ... drop connection");
      TelnetServer.available().stop();

    }
  }
 
}

void sendTelnetMsgln(String msg){
  sendTelnetMsg(msg + "\n\r");
  
}

void sendTelnetMsg(String msg){
    Serial.print(msg);
  for(telnetClients = 0; telnetClients < MAX_TELNET_CLIENTS; telnetClients++){ 
    if (TelnetClient[telnetClients] || TelnetClient[telnetClients].connected()){
      TelnetClient[telnetClients].print(msg);
      
    }
    
  }
  //Serial.print(msg);
  delay(10);  // to avoid strange characters left in buffer
  
}

void sendWiFiInformation(){
    sendTelnetMsg("SSID: ");
    sendTelnetMsgln(credentials.ssid);
    sendTelnetMsg("Passwort: ");
    sendTelnetMsgln(credentials.pass);
    sendTelnetMsg("Hostname: ");
    sendTelnetMsgln(credentials.hostnameESP);

    uint8_t status = WiFi.status();
    if(status == WL_IDLE_STATUS){
        sendTelnetMsgln("WiFi status ist momentan nicht verfügbar");
    
    }else if(status == WL_NO_SSID_AVAIL){
        sendTelnetMsgln("WiFi ist nicht in Reichweite");
    
    }else if(status == WL_CONNECTED){
        sendTelnetMsg("WiFi ist verbunden: ");
        sendTelnetMsgln(WiFi.localIP().toString());
    
    }else if(status == WL_CONNECT_FAILED){
        sendTelnetMsgln("WiFi Passwort falsch");
    
    }else if(status == WL_DISCONNECTED){
        sendTelnetMsgln("WiFi ist nicht in Station-mode");
    
    }else if(status == 10){
        sendTelnetMsgln("WiFi: Initialisierung läuft");
    
    }else{
        sendTelnetMsgln("WiFi: unbekannter Zustand");
    }
  
}

void sendMQTTInformation(){
    sendTelnetMsg("MQTT Server IP: ");
    sendTelnetMsgln(credentials.mqttServerIP);
    sendTelnetMsg("MQTT Topic: ");
    sendTelnetMsgln(credentials.mqttTopic);
    sendTelnetMsg("Client ID: ");
    sendTelnetMsgln(clientId);

    int status = mqtt.state();
    if(status == -4){
        sendTelnetMsgln("MQTT: connection timeout"); 
           
    }else if(status == -3){
        sendTelnetMsgln("MQTT: Verbindung verloren");
        
    }else if(status == -2){
        sendTelnetMsgln("MQTT: Verbindung fehlgeschlagen");
        
    }else if(status == -1){
        sendTelnetMsgln("MQTT: nicht verbunden");
        
    }else if(status == 0){
        sendTelnetMsgln("MQTT: Verbunden");
        
    }else if(status == 1){
        sendTelnetMsgln("MQTT: Falsches Protokoll");
        
    }else if(status == 2){
        sendTelnetMsgln("MQTT: ungültige ClientId");
        
    }else if(status == 3){
        sendTelnetMsgln("MQTT: Broker konnte die Verbindung nicht annehmen");
        
    }else if(status == 4){
        sendTelnetMsgln("MQTT: Benutzername oder Passwort falsch");
        
    }else if(status == 5){
        sendTelnetMsgln("MQTT: zugriff zum Broker verweigert");
        
    }else{
        sendTelnetMsgln("MQTT: unbekannter Fehler");
        
    }
    
}
