void handleConnections(){
    if(WiFi.status() == WL_CONNECTED){
        wifiNotConnectedSince = millis();
        WiFi.softAPdisconnect(true);
    
        if(!mqtt.connected() && (millis() - lastConnectToBroker > 60000 || connectFirstTime)){
            lastConnectToBroker = millis();
            connectFirstTime = false;
            char buffer[20];
            WiFi.localIP().toString().toCharArray(buffer, 20);  //Client ID kann erst hier gemacht werden, da sich ja die IP Adresse noch ändern könnte

            sprintf(clientId, "%s%s", credentials.hostnameESP, buffer);
            if(mqtt.connect(clientId)){     //Da dieser Verbindungsversuch sehr lange dauert führen wir ihn nur einmal pro Minute durch
                mqtt.subscribe(credentials.mqttTopic);
                WiFi.softAPdisconnect(true);
                
            }
        
        }
            
    }else if(!APStarted){
        APStarted = true;
        IPAddress local_IP(192, 168, 0, 1);
  		IPAddress gateway(192, 168, 0, 1);
 		IPAddress subnet(255, 255, 255, 0);
  		WiFi.softAPConfig(local_IP, gateway, subnet);
        WiFi.softAP(configSSID);
    
        mqtt.unsubscribe(credentials.mqttTopic);
        mqtt.disconnect();
    
    }else{
        if(millis() - wifiNotConnectedSince > 600000){ //Länger als 10 Minuten keine Verbindung
            ESP.restart();
        
        }
        
    }

}

void onMQTTMessageReceived(char* topic, byte* payload, unsigned int length){
    if(strcmp(topic, credentials.mqttTopic) == 0){
        if(memcmp(payload, "off", length) == 0){
            setOff = true;
            setColor(r, g, b, 0);
   
        }else if(memcmp(payload, "fade", length) == 0){
            fade = true;
            fadeState = 0;
   
        }else if(memcmp(payload, "red", length) == 0){
            setColor(255, 0, 0, brightness);
   
        }else if(memcmp(payload, "green", length) == 0){
            setColor(0, 255, 0, brightness);
   
        }else if(memcmp(payload, "blue", length) == 0){
            setColor(0, 0, 255, brightness);
   
        }else if(memcmp(payload, "white", length) == 0){
            setColor(255, 255, 255, brightness);
   
        }else if(memcmp(payload, "brighter", length) == 0){
            if(brightness <= 245){
                brightness = brightness + 10;
                setColor(r, g, b, brightness);
      
            }
   
        }else if(memcmp(payload, "darker", length) == 0){
            if(brightness >= 10){
                brightness = brightness - 10;
                setColor(r, g, b, brightness);
      
            }
   
        }else if(payload[length - 1] == '%'){
        	char temp[4];
        	memcpy(&temp[0], payload, length - 1);

        	brightness = atoi(temp);
        	setColor(r, g, b, brightness);
        	
        }
        else if(payload[0] == '#'){

			char temp[length + 1];
			memcpy(&temp[0], payload, length);
						
			int number = (int) strtol( &temp[1], NULL, 16);

			r = number >> 16;
			g = number >> 8 & 0xFF;
			b = number & 0xFF;
			
			setColor(r, g, b, brightness);
        	
        }else{
            Serial.print("MQTT Message nicht erkannt: ");
            char temp[25];
            memcpy(temp, payload, length);
            Serial.println(temp);
    
        }
        
    }
    
}

void handleFadeing(){
    
  if(fade && (millis() - fadeMillis >= 1000)){
    if(fadeState == 0){ //Initialzustand
      r = 255;
      g = 0;
      b = 0;
    }else if(fadeState <= 255){
      r = r - 1;
      g = g + 1;
      b = 0;
      
    }else if(fadeState <= 510){
      r = 0;
      g = g - 1;
      b = b + 1;
      
    }else if(fadeState <= 765){
      r = r + 1;
      g = 0;
      b = b - 1;
      
    }else{
      fadeState = 0;
      
    }
    
    analogWrite(pin_red, 255 - ((r * brightness) / 255));
    analogWrite(pin_green, 255 - ((g * brightness) / 255));
    analogWrite(pin_blue, 255 - ((b * brightness) / 255));
    fadeMillis = millis(); 
    fadeState = fadeState + 1;
    
  }
  
}


//https://github.com/esp8266/Arduino/issues/189
void setColor(byte red, byte green, byte blue, byte bright){

  //Sobald eine Farbe Gewählt wird, muss fade wieder ausgeschalten werden
  fade = false;

  dimmUpDown(red, green, blue, bright);

  r = red;
  g = green;
  b = blue;

}

void dimmUpDown(byte red, byte green, byte blue, byte bright){
  byte temp = 0;
  
  if(setOff && bright == 0){  //nur beim ausschalten
    temp = brightness;  // runter dimmen
     
  }else if(setOff && bright != 0){ //nur beim wiedereinschalten 
    temp = 0; //hoch dimmen
    setOff = false;
       
  }else{  //Farbwechsel ohne änderung der Helligkeit
    temp = bright;
    setOff = false;
    
  }
  
  while(true){
    analogWrite(pin_red, 255 - ((red * temp) / 255));
    analogWrite(pin_green, 255 - ((green * temp) / 255));
    analogWrite(pin_blue, 255 - ((blue * temp) / 255));
    
    if(temp < bright){
      temp = temp + 1;
    
    }else if(temp > bright){
      temp = temp - 1;

    }

    if(temp == bright){ //Abbruch bedingung für die Schleife
      analogWrite(pin_red, 255 - ((red * temp) / 255));
      analogWrite(pin_green, 255 - ((green * temp) / 255));
      analogWrite(pin_blue, 255 - ((blue * temp) / 255));
      break;
    }

    delay(10);
    
  }
}
